@@ -1,7 +1,5 @@
 package TAP::DOM::Entry;
-our $AUTHORITY = 'cpan:SCHWIGON';
-# ABSTRACT: Accessors for TAP::DOM line entries
-$TAP::DOM::Entry::VERSION = '0.93.2';
+
 use 5.006;
 use strict;
 use warnings;
@@ -15,7 +13,6 @@ use Class::XSAccessor
                         type
                         data
                         number
-                        severity
                         as_string
                         directive
                         description
@@ -55,8 +52,6 @@ __END__
 
 =pod
 
-=encoding UTF-8
-
 =head1 NAME
 
 TAP::DOM::Entry - Accessors for TAP::DOM line entries
@@ -116,17 +111,4 @@ accessors to all other entry fields.
 
 =head2 type
 
-=head2 severity
-
-=head1 AUTHOR
-
-Steffen Schwigon <ss5@renormalist.net>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2021 by Steffen Schwigon.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
