@@ -1,7 +1,5 @@
 package TAP::DOM::Summary;
-our $AUTHORITY = 'cpan:SCHWIGON';
-# ABSTRACT: Accessors for TAP::DOM summary part
-$TAP::DOM::Summary::VERSION = '0.93.2';
+
 use 5.006;
 use strict;
 use warnings;
@@ -32,12 +30,11 @@ __END__
 
 =pod
 
-=encoding UTF-8
-
 =head1 NAME
 
 TAP::DOM::Summary - Accessors for TAP::DOM summary part
 
+
 =head1 DESCRIPTION
 
 The C<summary> part covers aggregated results from TAP::Parser::Aggregator.
@@ -76,15 +73,4 @@ The C<summary> part covers aggregated results from TAP::Parser::Aggregator.
 
 =head2 wait
 
-=head1 AUTHOR
-
-Steffen Schwigon <ss5@renormalist.net>
-
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2021 by Steffen Schwigon.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
 =cut
